require 'erb/erbl'
require 'erb/erbu'
require 'drb'

module Div

  class Div
    include DRbUndumped
    extend ERbMethod

    @erb_fname=nil

    def self.set_erb(fname, dir=nil)
      if dir.nil?
	found = fname # default
	$:.each do |dir|
	  path = File::join(dir, fname)
	  if File::readable?(path)
	    found = path
	    break
	  end
	end
	@erb_fname = found
      else
	@erb_fname = File::join(dir, fname)
      end
      reload_erb
    end

    def self.reload_erb
      if @erb_fname
	begin
	  module_eval do
	    def_erb_method('to_html(context=nil)', @erb_fname)
	  end
	rescue SyntaxError
	end
      end
    end

    def initialize(session)
      @session = session
      @session.entry(self)
      @div_class = type.to_s
      @div_id = self.id.to_s
      @action = session.action
    end
    attr_reader :div_class, :div_id, :session

    def to_div(context)
      elem('div', {'class'=>@div_class, 'id'=>@div_id}) {
	begin
	  to_html(context)
	rescue
	  "<p>error! #{h($!)}</p>"
	end
      }
    end
    alias :to_s :to_div

    def to_html(context)
      ''
    end

    def do_request(context, params)
      cmd, = params['div_cmd']
      msg = 'do_' + cmd.to_s

      if respond_to?(msg) && msg != 'do_request'
	send(msg, context, params)
      else
	do_else(context, params)
      end
    end

    def do_else(context, params)
    end

    def action(context=nil)
      if context
	context.req_script_name + context.req_path_info
      else
	@action
      end
    end

    private
    def attr(opt)
      ary = opt.collect do |k, v|
	if v 
	  %Q!#{k}="#{h(v)}"!
	else
	  nil
	end
      end.compact
      return nil if ary.size == 0 
      ary.join(' ')
    end

    def elem(name, opt={})
      head = ["#{name}", attr(opt)].compact.join(" ")
      if block_given?
	%Q!<#{head}>\n#{yield}\n</#{name}>!
      else
	%Q!<#{head)} />!
      end  
    end

    def form(method, context)
      %Q!<form action="#{action(context)}" method="post">\n! +
	%Q!<input type="hidden" name="div_id" value="#{@div_id}" />\n! +
	%Q!<input type="hidden" name="div_cmd" value="#{h(method)}" />\n!
    end

    def href(method, param, context)
      param = {} unless param
      param['div_id'] = @div_id unless param['div_id']
      param['div_cmd'] = method unless param['div_cmd']

      ary = param.collect do |k, v|
	"#{u(k)}=#{u(v)}"
      end
      %Q!href="#{action(context)}?#{ary.join(';')}"!
    end

    def a(method, param, context)
      "<a #{href(method, param, context)}>"
    end

    def h(s)
      s.to_s.gsub(/&/n, '&amp;').gsub(/\"/n, '&quot;').gsub(/>/n, '&gt;').gsub(/</n, '&lt;')
    end

    def u(s)
      s.to_s.gsub(/([^ a-zA-Z0-9_.-]+)/n) do
	'%' + $1.unpack('H2' * $1.size).join('%').upcase
      end.tr(' ', '+')
    end
  end

  def reload_erb
    ObjectSpace.each_object(Class) do |o|
      if o.ancestors.include?(Div)
	o.reload_erb
      end
    end
  end
  module_function :reload_erb

  module KCode
    attr_reader(:lang, :charset)

    def kconv(s)
      return s unless @nkf
      return '' unless s
      NKF.nkf(@nkf, s)
    end

    def kconv_param(param)
      hash = {}
      param.each do |k, v|
	hash[k] = v.collect do |s|
	  kconv(s)
	end
      end
      hash
    end

    def kcode
      case $KCODE
      when /^[Ee]/
	@lang = 'ja'
	@charset = 'euc-jp'
	@nkf = '-edXm0'
      when /^[Ss]/
	@lang = 'ja'
	@charset = 'Shift_JIS'
	@nkf = '-sdXm0'
      else
	@lang = "en"
	@charset = 'us-ascii'
	@nkf = nil
      end
      require 'nkf' if @nkf
    end

    private :kcode

    module_function :lang, :charset, :kconv, :kcode
    kcode()
  end
end
